@@ -44,8 +44,8 @@ keybinding *keylist = NULL;
 
 extern Display *dpy;
 extern char  _STR[150], _STR2[250];
-extern short eventmode;
 extern Boolean pressmode;
+extern XCWindowData *areawin;
 
 /*--------------------------------------------------------------*/
 /* Key modifiers (convenience definitions)			*/
@@ -63,6 +63,8 @@ extern Boolean pressmode;
 #define BUTTON5 (Button5Mask << 16)
 #define HOLD (Mod2Mask << 16)
 
+#define ALL_WINDOWS (xcWidget)NULL
+
 /*--------------------------------------------------------------*/
 /* This list declares all the functions which can be bound to   */
 /* keys.  It must match the order of the enumeration listed in	*/
@@ -92,7 +94,8 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label", "Graphic",
    "Rescale", "Select Box", "Connectivity", "Continue Copy",
    "Continue Element", "Finish Copy", "Finish Element", "Finish",
-   "Cancel Segment", "Sim", "SPICE", "PCB", "SPICE Flat", "Text Delete Param",
+   "Cancel Last", "Sim", "SPICE", "PCB", "SPICE Flat", "Color",
+   "Text Delete Param", "Scale", "Change Style",
    NULL			/* sentinel */
 };
 
@@ -101,13 +104,14 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
 /* is bound to some function.					*/
 /*--------------------------------------------------------------*/
 
-Boolean ismacro(int keywstate)
+Boolean ismacro(xcWidget window, int keywstate)
 {
    keybinding *ksearch;
 
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
-      if (keywstate == ksearch->keywstate)
-	 return True;
+      if (ksearch->window == ALL_WINDOWS || ksearch->window == window)
+         if (keywstate == ksearch->keywstate)
+	    return True;
 
    return False;
 }
@@ -116,72 +120,66 @@ Boolean ismacro(int keywstate)
 /* Return the first key binding for the indicated function	*/
 /*--------------------------------------------------------------*/
 
-int firstbinding(int function)
-{
-   keybinding *ksearch;
-
-   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
-      if (function == ksearch->function)
-	 return ksearch->keywstate;
-
-   return -1;
-}
-
-/*--------------------------------------------------------------*/
-/* Return the nth function bound to the indicated keywstate.	*/
-/* Return -1 if no function is bound.				*/
-/*--------------------------------------------------------------*/
-
-int boundfunction(int keywstate, int n)
-{
-   int j;
-   keybinding *ksearch;
-
-   j = 0;
-   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
-      if (keywstate == ksearch->keywstate)
-	 if (j++ == n)
-	    return ksearch->function;
-
-   return -1;
-}
-
-/*--------------------------------------------------------------*/
-/* Return TRUE if the indicated key (keysym + bit-shifted state)*/
-/* is bound to the indicated function (called by name)		*/
-/*--------------------------------------------------------------*/
-
-Boolean isbound(int keywstate, int function)
-{
-   keybinding *ksearch;
-
-   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
-      if ((keywstate == ksearch->keywstate) && (function == ksearch->function))
-	 return True;
-
-   return False;
-}
-
-/*--------------------------------------------------------------*/
-/* Return TRUE if the key binding (keysym + bit-shifted state)  */
-/* is bound to a function name which matches the indicated	*/
-/* function name to "length" characters.  If the function name	*/
-/* includes a numerical suffix (e.g., "Page 1"), the suffix is	*/
-/* converted to an integer and passed back in "retnum".		*/
-/*--------------------------------------------------------------*/
-
-Boolean isnbound(int keywstate, int function, short *retnum)
+int firstbinding(xcWidget window, int function)
 {
    keybinding *ksearch;
+   int keywstate = -1;
 
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
-      if ((keywstate == ksearch->keywstate) && (function == ksearch->function)) {
-	 if (retnum != NULL)
-	    *retnum = (ksearch->value);
-	 return True;
+      if (ksearch->window == window)
+	 return ksearch->keywstate;
+      else if (ksearch->window == ALL_WINDOWS)
+	 keywstate = ksearch->keywstate;
+   }
+   return keywstate;
+}
+
+/*--------------------------------------------------------------*/
+/* Find the first function bound to the indicated key that is	*/
+/* compatible with the current state (eventmode).  Window-	*/
+/* specific bindings shadow ALL_WINDOWS bindings.  Return the	*/
+/* function number if found, or -1 if no compatible functions	*/
+/* are bound to the key state.					*/
+/*--------------------------------------------------------------*/
+
+int boundfunction(xcWidget window, int keywstate, short *retnum)
+{
+   keybinding *ksearch;
+   int tempfunc = -1;
+
+   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
+      if (keywstate == ksearch->keywstate) {
+	 if (compatible_function(ksearch->function)) {
+	    if (ksearch->window == window) {
+	       if (retnum != NULL) *retnum = (ksearch->value);
+	       return ksearch->function;
+	    }
+	    else if (ksearch->window == ALL_WINDOWS) {
+	       if (retnum != NULL) *retnum = (ksearch->value);
+	       tempfunc = ksearch->function;
+	    }
+	 }
       }
    }
-   return False;
+   return tempfunc;
+}
+
+/*--------------------------------------------------------------*/
+/* Check if an entry exists for a given key-function pair	*/
+/*--------------------------------------------------------------*/
+
+int isbound(xcWidget window, int keywstate, int function, short value)
+{
+   keybinding *ksearch;
+
+   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
+      if (keywstate == ksearch->keywstate && function == ksearch->function)
+	 if (window == ALL_WINDOWS || window == ksearch->window ||
+		ksearch->window == ALL_WINDOWS)
+	    if (value == -1 || value == ksearch->value || ksearch->value == -1)
+	       return TRUE;
+
+   return FALSE;
 }
 
 /*--------------------------------------------------------------*/
@@ -309,12 +307,10 @@ int string_to_key(const char *keystring)
 
 /*--------------------------------------------------------------*/
 /* Convert a function into a string representing all of its	*/
-/* key bindings, or convert a key into a string representing	*/
-/* all of its function bindings.  One of the arguments is	*/
-/* valid, and the other should be -1.				*/
+/* key bindings.						*/
 /*--------------------------------------------------------------*/
 
-char *binding_to_string(int keywstate, int function)
+char *function_binding_to_string(xcWidget window, int function)
 {
    keybinding *ksearch;
    char *retstr, *tmpstr;
@@ -323,19 +319,18 @@ char *binding_to_string(int keywstate, int function)
    retstr = (char *)malloc(1);
    retstr[0] = '\0';
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
-      if (function == ksearch->function || keywstate == ksearch->keywstate) {
-	 if (function < 0)
-	    tmpstr = function_names[ksearch->function];
-	 else
+      if (function == ksearch->function) {
+         if (ksearch->window == ALL_WINDOWS || ksearch->window == window) {
 	    tmpstr = key_to_string(ksearch->keywstate);
-	 if (tmpstr != NULL) {
-	    retstr = (char *)realloc(retstr, strlen(retstr) + strlen(tmpstr) + 
-		((first) ? 1 : 3));
-	    if (!first) strcat(retstr, ", ");
-	    strcat(retstr, tmpstr);
-	    if (function >= 0) free(tmpstr);
+	    if (tmpstr != NULL) {
+	       retstr = (char *)realloc(retstr, strlen(retstr) + strlen(tmpstr) + 
+		   ((first) ? 1 : 3));
+	       if (!first) strcat(retstr, ", ");
+	       strcat(retstr, tmpstr);
+	       free(tmpstr);
+	    }
+	    first = False;
 	 }
-	 first = False;
       }
    }
    if (retstr[0] == '\0') {
@@ -345,6 +340,67 @@ char *binding_to_string(int keywstate, int function)
    return(retstr);
 }
 
+/*--------------------------------------------------------------*/
+/* Convert a key into a string representing all of its function	*/
+/* bindings.							*/
+/*--------------------------------------------------------------*/
+
+char *key_binding_to_string(xcWidget window, int keywstate)
+{
+   keybinding *ksearch;
+   char *retstr, *tmpstr;
+   Bool first = True;
+
+   retstr = (char *)malloc(1);
+   retstr[0] = '\0';
+   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
+      if (keywstate == ksearch->keywstate) {
+         if (ksearch->window == ALL_WINDOWS || ksearch->window == window) {
+	    tmpstr = function_names[ksearch->function];
+	    if (tmpstr != NULL) {
+	       retstr = (char *)realloc(retstr, strlen(retstr) + strlen(tmpstr) + 
+		   ((first) ? 1 : 3));
+	       if (!first) strcat(retstr, ", ");
+	       strcat(retstr, tmpstr);
+	    }
+	    first = False;
+	 }
+      }
+   }
+   if (retstr[0] == '\0') {
+      retstr = (char *)realloc(retstr, 10);
+      strcat(retstr, "<unbound>");
+   }
+   return(retstr);
+}
+
+/*--------------------------------------------------------------*/
+/* Return an allocated string name of the function that		*/
+/* is bound to the indicated key state for the indicated	*/
+/* window and compattible with the current event mode.		*/
+/*--------------------------------------------------------------*/
+
+char *compat_key_to_string(xcWidget window, int keywstate)
+{
+   char *retstr, *tmpstr;
+   int function;
+
+   function = boundfunction(window, keywstate, NULL);
+   tmpstr = func_to_string(function);
+
+   /* Pass back "Nothing" for unbound key states, since a	*/
+   /* wrapper script may want to use the result as an action.	*/
+
+   if (tmpstr == NULL) {
+      retstr = (char *)malloc(8);
+      strcpy(retstr, "Nothing");
+   }
+   else
+      retstr = strdup(tmpstr);
+
+   return(retstr);
+}
+
 /*--------------------------------------------------------------*/
 /* Convert a key sym into a string				*/
 /*--------------------------------------------------------------*/
@@ -353,9 +409,11 @@ char *key_to_string(int keywstate)
 {
    static char hex[17] = "0123456789ABCDEF";
    char *kptr, *str = NULL;
-   KeySym ks = keywstate & 0xffff;
-   int kmod = keywstate >> 16;
+   KeySym ks;
+   int kmod;
 
+   ks = keywstate & 0xffff;
+   kmod = keywstate >> 16;
    if (ks != NoSymbol) str = XKeysymToString(ks);
 
    kptr = (char *)malloc(32);
@@ -402,60 +460,69 @@ void printeditbindings()
 
    _STR2[0] = '\0';
 
-   tstr = key_to_string(firstbinding(XCF_Edit_Break));
+   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Break));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
    strcat(_STR2, func_to_string(XCF_Edit_Break));
    strcat(_STR2, ", ");
    free(tstr);
 
-   tstr = key_to_string(firstbinding(XCF_Edit_Delete));
+   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Delete));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
    strcat(_STR2, func_to_string(XCF_Edit_Delete));
    strcat(_STR2, ", ");
    free(tstr);
 
-   tstr = key_to_string(firstbinding(XCF_Edit_Insert));
+   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Insert));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
    strcat(_STR2, func_to_string(XCF_Edit_Insert));
    strcat(_STR2, ", ");
    free(tstr);
 
-   tstr = key_to_string(firstbinding(XCF_Edit_Param));
+   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Param));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
    strcat(_STR2, func_to_string(XCF_Edit_Param));
    strcat(_STR2, ", ");
    free(tstr);
 
-   tstr = key_to_string(firstbinding(XCF_Edit_Next));
+   tstr = key_to_string(firstbinding(areawin->area, XCF_Edit_Next));
    strcat(_STR2, tstr);
    strcat(_STR2, "=");
    strcat(_STR2, func_to_string(XCF_Edit_Next));
    free(tstr);
 
-   Wprintf(_STR2);
+   /* Use W3printf().  In the Tcl version, this prints to the	*/
+   /* message window but does not duplicate the output to 	*/
+   /* stdout, where it would be just an annoyance.		*/
+
+   W3printf("%s", _STR2);
 }
 
 /*--------------------------------------------------------------*/
 /* Remove a key binding from the list				*/
+/*								*/
+/* Note:  This routine needs to correctly handle ALL_WINDOWS	*/
+/* bindings that shadow specific window bindings.		*/
 /*--------------------------------------------------------------*/
 
-int remove_binding(int keywstate, int function)
+int remove_binding(xcWidget window, int keywstate, int function)
 {
    keybinding *ksearch, *klast = NULL;
 
    for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding) {
-      if ((function == ksearch->function)
+      if (window == ALL_WINDOWS || window == ksearch->window) {
+         if ((function == ksearch->function)
 		&& (keywstate == ksearch->keywstate)) {
-	 if (klast == NULL)
-	    keylist = ksearch->nextbinding;
-	 else
-	    klast->nextbinding = ksearch->nextbinding;
-	 free(ksearch);
-	 return 0;
+	    if (klast == NULL)
+	       keylist = ksearch->nextbinding;
+	    else
+	       klast->nextbinding = ksearch->nextbinding;
+	    free(ksearch);
+	    return 0;
+         }
       }
       klast = ksearch;
    }
@@ -466,15 +533,14 @@ int remove_binding(int keywstate, int function)
 /* Wrapper for remove_binding					*/
 /*--------------------------------------------------------------*/
 
-void remove_keybinding(const char *keystring, const char *fstring)
+void remove_keybinding(xcWidget window, const char *keystring, const char *fstring)
 {
    int function = string_to_func(fstring, NULL);
    int keywstate = string_to_key(keystring);
 
-   if ((function < 0) || (remove_binding(keywstate, function) < 0)) {
-      sprintf(_STR2, "Key binding \'%s\' to \'%s\' does not exist in list.",
+   if ((function < 0) || (remove_binding(window, keywstate, function) < 0)) {
+      Wprintf("Key binding \'%s\' to \'%s\' does not exist in list.",
 		keystring, fstring);
-      Wprintf(_STR2);
    }
 }
 
@@ -482,17 +548,18 @@ void remove_keybinding(const char *keystring, const char *fstring)
 /* Add a key binding to the list				*/
 /*--------------------------------------------------------------*/
 
-int add_vbinding(int keywstate, int function, short value)
+int add_vbinding(xcWidget window, int keywstate, int function, short value)
 {
    keybinding *newbinding;
 
    /* If key is already bound to the function, ignore it */
 
-   if (isbound(keywstate, function)) return 1;
-
+   if (isbound(window, keywstate, function, value)) return 1;
+	   
    /* Add the new key binding */
 
    newbinding = (keybinding *)malloc(sizeof(keybinding));
+   newbinding->window = window;
    newbinding->keywstate = keywstate;
    newbinding->function = function;
    newbinding->value = value;
@@ -505,16 +572,16 @@ int add_vbinding(int keywstate, int function, short value)
 /* Wrapper function for key binding without any values		*/
 /*--------------------------------------------------------------*/
 
-int add_binding(int keywstate, int function)
+int add_binding(xcWidget window, int keywstate, int function)
 {
-   return add_vbinding(keywstate, function, (short)-1);
+   return add_vbinding(window, keywstate, function, (short)-1);
 }
 
 /*--------------------------------------------------------------*/
 /* Wrapper function for key binding with function as string	*/
 /*--------------------------------------------------------------*/
 
-int add_keybinding(const char *keystring, const char *fstring)
+int add_keybinding(xcWidget window, const char *keystring, const char *fstring)
 {
    short value = -1;
    int function = string_to_func(fstring, &value);
@@ -523,7 +590,7 @@ int add_keybinding(const char *keystring, const char *fstring)
    if (function < 0)
       return -1;
    else
-      return add_vbinding(keywstate, function, value);
+      return add_vbinding(window, keywstate, function, value);
 }
 
 /*--------------------------------------------------------------*/
@@ -534,186 +601,197 @@ int add_keybinding(const char *keystring, const char *fstring)
 
 void default_keybindings()
 {
-   add_vbinding(XK_1, XCF_Page, 1);
-   add_vbinding(XK_2, XCF_Page, 2);
-   add_vbinding(XK_3, XCF_Page, 3);
-   add_vbinding(XK_4, XCF_Page, 4);
-   add_vbinding(XK_5, XCF_Page, 5);
-   add_vbinding(XK_6, XCF_Page, 6);
-   add_vbinding(XK_7, XCF_Page, 7);
-   add_vbinding(XK_8, XCF_Page, 8);
-   add_vbinding(XK_9, XCF_Page, 9);
-   add_vbinding(XK_0, XCF_Page, 10);
+   add_vbinding(ALL_WINDOWS, XK_1, XCF_Page, 1);
+   add_vbinding(ALL_WINDOWS, XK_2, XCF_Page, 2);
+   add_vbinding(ALL_WINDOWS, XK_3, XCF_Page, 3);
+   add_vbinding(ALL_WINDOWS, XK_4, XCF_Page, 4);
+   add_vbinding(ALL_WINDOWS, XK_5, XCF_Page, 5);
+   add_vbinding(ALL_WINDOWS, XK_6, XCF_Page, 6);
+   add_vbinding(ALL_WINDOWS, XK_7, XCF_Page, 7);
+   add_vbinding(ALL_WINDOWS, XK_8, XCF_Page, 8);
+   add_vbinding(ALL_WINDOWS, XK_9, XCF_Page, 9);
+   add_vbinding(ALL_WINDOWS, XK_0, XCF_Page, 10);
 
-   add_vbinding(SHIFT | XK_KP_1, XCF_Justify, 0);
-   add_vbinding(SHIFT | XK_KP_2, XCF_Justify, 1);
-   add_vbinding(SHIFT | XK_KP_3, XCF_Justify, 2);
-   add_vbinding(SHIFT | XK_KP_4, XCF_Justify, 3);
-   add_vbinding(SHIFT | XK_KP_5, XCF_Justify, 4);
-   add_vbinding(SHIFT | XK_KP_6, XCF_Justify, 5);
-   add_vbinding(SHIFT | XK_KP_7, XCF_Justify, 6);
-   add_vbinding(SHIFT | XK_KP_8, XCF_Justify, 7);
-   add_vbinding(SHIFT | XK_KP_9, XCF_Justify, 8);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_1, XCF_Justify, 0);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_2, XCF_Justify, 1);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_3, XCF_Justify, 2);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_4, XCF_Justify, 3);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_5, XCF_Justify, 4);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_6, XCF_Justify, 5);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_7, XCF_Justify, 6);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_8, XCF_Justify, 7);
+   add_vbinding(ALL_WINDOWS, SHIFT | XK_KP_9, XCF_Justify, 8);
 
-   add_vbinding(XK_KP_End, XCF_Justify, 0);
-   add_vbinding(XK_KP_Down, XCF_Justify, 1);
-   add_vbinding(XK_KP_Next, XCF_Justify, 2);
-   add_vbinding(XK_KP_Left, XCF_Justify, 3);
-   add_vbinding(XK_KP_Begin, XCF_Justify, 4);
-   add_vbinding(XK_KP_Right, XCF_Justify, 5);
-   add_vbinding(XK_KP_Home, XCF_Justify, 6);
-   add_vbinding(XK_KP_Up, XCF_Justify, 7);
-   add_vbinding(XK_KP_Prior, XCF_Justify, 8);
+   add_vbinding(ALL_WINDOWS, XK_KP_End, XCF_Justify, 0);
+   add_vbinding(ALL_WINDOWS, XK_KP_Down, XCF_Justify, 1);
+   add_vbinding(ALL_WINDOWS, XK_KP_Next, XCF_Justify, 2);
+   add_vbinding(ALL_WINDOWS, XK_KP_Left, XCF_Justify, 3);
+   add_vbinding(ALL_WINDOWS, XK_KP_Begin, XCF_Justify, 4);
+   add_vbinding(ALL_WINDOWS, XK_KP_Right, XCF_Justify, 5);
+   add_vbinding(ALL_WINDOWS, XK_KP_Home, XCF_Justify, 6);
+   add_vbinding(ALL_WINDOWS, XK_KP_Up, XCF_Justify, 7);
+   add_vbinding(ALL_WINDOWS, XK_KP_Prior, XCF_Justify, 8);
 
-   add_binding(XK_Delete, XCF_Text_Delete);
-   add_binding(XK_Return, XCF_Text_Return);
-   add_binding(BUTTON1, XCF_Text_Return);
-   add_binding(XK_BackSpace, XCF_Text_Delete);
-   add_binding(XK_Left, XCF_Text_Left);
-   add_binding(XK_Right, XCF_Text_Right);
-   add_binding(XK_Up, XCF_Text_Up);
-   add_binding(XK_Down, XCF_Text_Down);
-   add_binding(ALT | XK_x, XCF_Text_Split);
-   add_binding(XK_Home, XCF_Text_Home);
-   add_binding(XK_End, XCF_Text_End);
-   add_binding(XK_Tab, XCF_TabForward);
-   add_binding(SHIFT | XK_Tab, XCF_TabBackward);
+   add_binding(ALL_WINDOWS, XK_Delete, XCF_Text_Delete);
+   add_binding(ALL_WINDOWS, XK_Return, XCF_Text_Return);
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Text_Return);
+   add_binding(ALL_WINDOWS, XK_BackSpace, XCF_Text_Delete);
+   add_binding(ALL_WINDOWS, XK_Left, XCF_Text_Left);
+   add_binding(ALL_WINDOWS, XK_Right, XCF_Text_Right);
+   add_binding(ALL_WINDOWS, XK_Up, XCF_Text_Up);
+   add_binding(ALL_WINDOWS, XK_Down, XCF_Text_Down);
+   add_binding(ALL_WINDOWS, ALT | XK_x, XCF_Text_Split);
+   add_binding(ALL_WINDOWS, XK_Home, XCF_Text_Home);
+   add_binding(ALL_WINDOWS, XK_End, XCF_Text_End);
+   add_binding(ALL_WINDOWS, XK_Tab, XCF_TabForward);
+   add_binding(ALL_WINDOWS, SHIFT | XK_Tab, XCF_TabBackward);
 #ifdef XK_ISO_Left_Tab
-   add_binding(SHIFT | XK_ISO_Left_Tab, XCF_TabBackward);
+   add_binding(ALL_WINDOWS, SHIFT | XK_ISO_Left_Tab, XCF_TabBackward);
 #endif
-   add_binding(ALT | XK_Tab, XCF_TabStop);
-   add_binding(XK_KP_Add, XCF_Superscript);
-   add_binding(XK_KP_Subtract, XCF_Subscript);
-   add_binding(XK_KP_Enter, XCF_Normalscript);
-   add_binding(ALT | XK_f, XCF_Nextfont);
-   add_binding(ALT | XK_b, XCF_Boldfont);
-   add_binding(ALT | XK_i, XCF_Italicfont);
-   add_binding(ALT | XK_n, XCF_Normalfont);
-   add_binding(ALT | XK_u, XCF_Underline);
-   add_binding(ALT | XK_o, XCF_Overline);
-   add_binding(ALT | XK_e, XCF_ISO_Encoding);
-   add_binding(ALT | XK_Return, XCF_Linebreak);
-   add_binding(ALT | XK_h, XCF_Halfspace);
-   add_binding(ALT | XK_q, XCF_Quarterspace);
-   add_binding(XK_backslash, XCF_Special);
-   add_binding(ALT | XK_c, XCF_Special);
-   add_binding(ALT | XK_p, XCF_Parameter);
-   add_binding(XK_p, XCF_Edit_Param);
-   add_binding(XK_x, XCF_Edit_Break);
-   add_binding(XK_d, XCF_Edit_Delete);
-   add_binding(XK_Delete, XCF_Edit_Delete);
-   add_binding(XK_i, XCF_Edit_Insert);
-   add_binding(XK_Insert, XCF_Edit_Insert);
-   add_binding(XK_e, XCF_Edit_Next);
-   add_binding(BUTTON1, XCF_Edit_Next);
-   add_binding(XK_A, XCF_Attach);
-   add_binding(XK_V, XCF_Virtual);
-   add_binding(XK_l, XCF_Next_Library);
-   add_binding(XK_L, XCF_Library_Directory);
-   add_binding(XK_c, XCF_Library_Copy);
-   add_binding(XK_E, XCF_Library_Edit);
-   add_binding(XK_D, XCF_Library_Delete);
-   add_binding(XK_C, XCF_Library_Duplicate);
-   add_binding(XK_H, XCF_Library_Hide);
-   add_binding(XK_V, XCF_Library_Virtual);
-   add_binding(XK_M, XCF_Library_Move);
-   add_binding(XK_P, XCF_Page_Directory);
-   add_binding(XK_less, XCF_Library_Pop);
-   add_binding(HOLD | BUTTON1, XCF_Library_Pop);
-   add_binding(XK_h, XCF_Help);
-   add_binding(XK_question, XCF_Help);
-   add_binding(XK_space, XCF_Redraw);
-   add_binding(XK_Redo, XCF_Redraw);
-   add_binding(XK_Undo, XCF_Redraw);
-   add_binding(XK_Home, XCF_View);
-   add_binding(XK_v, XCF_View);
-   add_binding(XK_Z, XCF_Zoom_In);
-   add_binding(XK_z, XCF_Zoom_Out);
-   add_vbinding(XK_p, XCF_Pan, 0);
-   add_binding(XK_plus, XCF_Double_Snap);
-   add_binding(XK_minus, XCF_Halve_Snap);
-   add_vbinding(XK_Left, XCF_Pan, 1);
-   add_vbinding(XK_Right, XCF_Pan, 2);
-   add_vbinding(XK_Up, XCF_Pan, 3);
-   add_vbinding(XK_Down, XCF_Pan, 4);
-   add_binding(XK_W, XCF_Write);
-   add_vbinding(XK_O, XCF_Rotate, -5);
-   add_vbinding(XK_o, XCF_Rotate, 5);
-   add_vbinding(XK_R, XCF_Rotate, -15);
-   add_vbinding(XK_r, XCF_Rotate, 15);
-   add_binding(XK_f, XCF_Flip_X);
-   add_binding(XK_F, XCF_Flip_Y);
-   add_binding(XK_S, XCF_Snap);
-   add_binding(XK_less, XCF_Pop);
-   add_binding(XK_greater, XCF_Push);
-   add_binding(XK_Delete, XCF_Delete);
-   add_binding(XK_d, XCF_Delete);
-   add_binding(XK_F19, XCF_Select);
-   add_binding(XK_b, XCF_Box);
-   add_binding(XK_a, XCF_Arc);
-   add_binding(XK_t, XCF_Text);
-   add_binding(XK_X, XCF_Exchange);
-   add_binding(XK_c, XCF_Copy);
-   add_binding(XK_j, XCF_Join);
-   add_binding(XK_J, XCF_Unjoin);
-   add_binding(XK_s, XCF_Spline);
-   add_binding(XK_e, XCF_Edit);
-   add_binding(XK_u, XCF_Undo);
-   add_binding(XK_U, XCF_Redo);
-   add_binding(XK_M, XCF_Select_Save);
-   add_binding(XK_m, XCF_Select_Save);
-   add_binding(XK_x, XCF_Unselect);
-   add_binding(XK_bar, XCF_Dashed);
-   add_binding(XK_colon, XCF_Dotted);
-   add_binding(XK_underscore, XCF_Solid);
-   add_binding(XK_percent, XCF_Prompt);
-   add_binding(XK_period, XCF_Dot);
-   add_binding(BUTTON1, XCF_Wire);
-   add_binding(XK_w, XCF_Wire);
-   add_binding(CTRL | ALT | XK_q, XCF_Exit);
-   add_binding(HOLD | BUTTON1, XCF_Move);
-   add_binding(BUTTON1, XCF_Continue_Element);
-   add_binding(BUTTON1, XCF_Continue_Copy);
-   add_binding(BUTTON1, XCF_Finish);
-   add_binding(XK_Escape, XCF_Cancel);
-   add_binding(ALT | XK_r, XCF_Rescale);
-   add_binding(ALT | XK_s, XCF_SnapTo);
-   add_binding(ALT | XK_q, XCF_Netlist);
-   add_binding(XK_slash, XCF_Swap);
-   add_binding(XK_T, XCF_Pin_Label);
-   add_binding(XK_G, XCF_Pin_Global);
-   add_binding(XK_I, XCF_Info_Label);
-   add_binding(ALT | XK_w, XCF_Connectivity);
+   add_binding(ALL_WINDOWS, ALT | XK_Tab, XCF_TabStop);
+   add_binding(ALL_WINDOWS, XK_KP_Add, XCF_Superscript);
+   add_binding(ALL_WINDOWS, XK_KP_Subtract, XCF_Subscript);
+   add_binding(ALL_WINDOWS, XK_KP_Enter, XCF_Normalscript);
+   add_vbinding(ALL_WINDOWS, ALT | XK_f, XCF_Font, 1000);
+   add_binding(ALL_WINDOWS, ALT | XK_b, XCF_Boldfont);
+   add_binding(ALL_WINDOWS, ALT | XK_i, XCF_Italicfont);
+   add_binding(ALL_WINDOWS, ALT | XK_n, XCF_Normalfont);
+   add_binding(ALL_WINDOWS, ALT | XK_u, XCF_Underline);
+   add_binding(ALL_WINDOWS, ALT | XK_o, XCF_Overline);
+   add_binding(ALL_WINDOWS, ALT | XK_e, XCF_ISO_Encoding);
+   add_binding(ALL_WINDOWS, ALT | XK_Return, XCF_Linebreak);
+   add_binding(ALL_WINDOWS, ALT | XK_h, XCF_Halfspace);
+   add_binding(ALL_WINDOWS, ALT | XK_q, XCF_Quarterspace);
+   add_binding(ALL_WINDOWS, XK_backslash, XCF_Special);
+   add_binding(ALL_WINDOWS, ALT | XK_c, XCF_Special);
+   add_binding(ALL_WINDOWS, ALT | XK_p, XCF_Parameter);
+   add_binding(ALL_WINDOWS, XK_p, XCF_Edit_Param);
+   add_binding(ALL_WINDOWS, XK_x, XCF_Edit_Break);
+   add_binding(ALL_WINDOWS, XK_d, XCF_Edit_Delete);
+   add_binding(ALL_WINDOWS, XK_Delete, XCF_Edit_Delete);
+   add_binding(ALL_WINDOWS, XK_i, XCF_Edit_Insert);
+   add_binding(ALL_WINDOWS, XK_Insert, XCF_Edit_Insert);
+   add_binding(ALL_WINDOWS, XK_e, XCF_Edit_Next);
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Edit_Next);
+   add_binding(ALL_WINDOWS, XK_A, XCF_Attach);
+   add_binding(ALL_WINDOWS, XK_V, XCF_Virtual);
+   add_binding(ALL_WINDOWS, XK_l, XCF_Next_Library);
+   add_binding(ALL_WINDOWS, XK_L, XCF_Library_Directory);
+   add_binding(ALL_WINDOWS, XK_c, XCF_Library_Copy);
+   add_binding(ALL_WINDOWS, XK_E, XCF_Library_Edit);
+   add_binding(ALL_WINDOWS, XK_D, XCF_Library_Delete);
+   add_binding(ALL_WINDOWS, XK_C, XCF_Library_Duplicate);
+   add_binding(ALL_WINDOWS, XK_H, XCF_Library_Hide);
+   add_binding(ALL_WINDOWS, XK_V, XCF_Library_Virtual);
+   add_binding(ALL_WINDOWS, XK_M, XCF_Library_Move);
+   add_binding(ALL_WINDOWS, XK_P, XCF_Page_Directory);
+   add_binding(ALL_WINDOWS, XK_less, XCF_Library_Pop);
+   add_binding(ALL_WINDOWS, HOLD | BUTTON1, XCF_Library_Pop);
+   add_binding(ALL_WINDOWS, XK_h, XCF_Help);
+   add_binding(ALL_WINDOWS, XK_question, XCF_Help);
+   add_binding(ALL_WINDOWS, XK_space, XCF_Redraw);
+   add_binding(ALL_WINDOWS, XK_Redo, XCF_Redraw);
+   add_binding(ALL_WINDOWS, XK_Undo, XCF_Redraw);
+   add_binding(ALL_WINDOWS, XK_Home, XCF_View);
+   add_binding(ALL_WINDOWS, XK_v, XCF_View);
+   add_binding(ALL_WINDOWS, XK_Z, XCF_Zoom_In);
+   add_binding(ALL_WINDOWS, XK_z, XCF_Zoom_Out);
+   add_vbinding(ALL_WINDOWS, XK_p, XCF_Pan, 0);
+   add_binding(ALL_WINDOWS, XK_plus, XCF_Double_Snap);
+   add_binding(ALL_WINDOWS, XK_minus, XCF_Halve_Snap);
+   add_vbinding(ALL_WINDOWS, XK_Left, XCF_Pan, 1);
+   add_vbinding(ALL_WINDOWS, XK_Right, XCF_Pan, 2);
+   add_vbinding(ALL_WINDOWS, XK_Up, XCF_Pan, 3);
+   add_vbinding(ALL_WINDOWS, XK_Down, XCF_Pan, 4);
+   add_binding(ALL_WINDOWS, XK_W, XCF_Write);
+   add_vbinding(ALL_WINDOWS, XK_O, XCF_Rotate, -5);
+   add_vbinding(ALL_WINDOWS, XK_o, XCF_Rotate, 5);
+   add_vbinding(ALL_WINDOWS, XK_R, XCF_Rotate, -15);
+   add_vbinding(ALL_WINDOWS, XK_r, XCF_Rotate, 15);
+   add_binding(ALL_WINDOWS, XK_f, XCF_Flip_X);
+   add_binding(ALL_WINDOWS, XK_F, XCF_Flip_Y);
+   add_binding(ALL_WINDOWS, XK_S, XCF_Snap);
+   add_binding(ALL_WINDOWS, XK_less, XCF_Pop);
+   add_binding(ALL_WINDOWS, XK_greater, XCF_Push);
+   add_binding(ALL_WINDOWS, XK_Delete, XCF_Delete);
+   add_binding(ALL_WINDOWS, XK_d, XCF_Delete);
+   add_binding(ALL_WINDOWS, XK_F19, XCF_Select);
+   add_binding(ALL_WINDOWS, XK_b, XCF_Box);
+   add_binding(ALL_WINDOWS, XK_a, XCF_Arc);
+   add_binding(ALL_WINDOWS, XK_t, XCF_Text);
+   add_binding(ALL_WINDOWS, XK_X, XCF_Exchange);
+   add_binding(ALL_WINDOWS, XK_c, XCF_Copy);
+   add_binding(ALL_WINDOWS, XK_j, XCF_Join);
+   add_binding(ALL_WINDOWS, XK_J, XCF_Unjoin);
+   add_binding(ALL_WINDOWS, XK_s, XCF_Spline);
+   add_binding(ALL_WINDOWS, XK_e, XCF_Edit);
+   add_binding(ALL_WINDOWS, XK_u, XCF_Undo);
+   add_binding(ALL_WINDOWS, XK_U, XCF_Redo);
+   add_binding(ALL_WINDOWS, XK_M, XCF_Select_Save);
+   add_binding(ALL_WINDOWS, XK_m, XCF_Select_Save);
+   add_binding(ALL_WINDOWS, XK_x, XCF_Unselect);
+   add_binding(ALL_WINDOWS, XK_bar, XCF_Dashed);
+   add_binding(ALL_WINDOWS, XK_colon, XCF_Dotted);
+   add_binding(ALL_WINDOWS, XK_underscore, XCF_Solid);
+   add_binding(ALL_WINDOWS, XK_percent, XCF_Prompt);
+   add_binding(ALL_WINDOWS, XK_period, XCF_Dot);
+#ifndef TCL_WRAPPER
+   /* TCL_WRAPPER version req's binding to specific windows */
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Wire);
+#endif
+   add_binding(ALL_WINDOWS, XK_w, XCF_Wire);
+   add_binding(ALL_WINDOWS, CTRL | ALT | XK_q, XCF_Exit);
+   add_binding(ALL_WINDOWS, HOLD | BUTTON1, XCF_Move);
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Continue_Element);
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Continue_Copy);
+   add_binding(ALL_WINDOWS, BUTTON1, XCF_Finish);
+   add_binding(ALL_WINDOWS, XK_Escape, XCF_Cancel);
+   add_binding(ALL_WINDOWS, ALT | XK_r, XCF_Rescale);
+   add_binding(ALL_WINDOWS, ALT | XK_s, XCF_SnapTo);
+   add_binding(ALL_WINDOWS, ALT | XK_q, XCF_Netlist);
+   add_binding(ALL_WINDOWS, XK_slash, XCF_Swap);
+   add_binding(ALL_WINDOWS, XK_T, XCF_Pin_Label);
+   add_binding(ALL_WINDOWS, XK_G, XCF_Pin_Global);
+   add_binding(ALL_WINDOWS, XK_I, XCF_Info_Label);
+   add_binding(ALL_WINDOWS, ALT | XK_w, XCF_Connectivity);
 
 /* These are for test purposes only.  Menu selection is	*/
 /* preferred.						*/
 
-/* add_binding(ALT | XK_d, XCF_Sim);		*/
-/* add_binding(ALT | XK_a, XCF_SPICE);		*/
-/* add_binding(ALT | XK_f, XCF_SPICEflat);	*/
-/* add_binding(ALT | XK_p, XCF_PCB);		*/
+/* add_binding(ALL_WINDOWS, ALT | XK_d, XCF_Sim);		*/
+/* add_binding(ALL_WINDOWS, ALT | XK_a, XCF_SPICE);		*/
+/* add_binding(ALL_WINDOWS, ALT | XK_f, XCF_SPICEflat);	*/
+/* add_binding(ALL_WINDOWS, ALT | XK_p, XCF_PCB);		*/
+
+   /* Avoid spurious Num_Lock messages */
+   add_binding(ALL_WINDOWS, XK_Num_Lock, XCF_Nothing);
 
    /* 2-button vs. 3-button mouse bindings (set with -2	*/
    /* commandline option; 3-button bindings default)	*/
 
    if (pressmode) {
-      add_binding(BUTTON3, XCF_Text_Return);
-      add_binding(BUTTON3, XCF_Select);
-      add_binding(HOLD | BUTTON3, XCF_SelectBox);
-      add_binding(BUTTON3, XCF_Finish_Element);
-      add_binding(BUTTON3, XCF_Finish_Copy);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Text_Return);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Select);
+      add_binding(ALL_WINDOWS, HOLD | BUTTON3, XCF_SelectBox);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Finish_Element);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Finish_Copy);
 
-      add_binding(XK_BackSpace, XCF_Cancel_Last);
-      add_binding(XK_BackSpace, XCF_Cancel);
+      add_binding(ALL_WINDOWS, XK_BackSpace, XCF_Cancel_Last);
+      add_binding(ALL_WINDOWS, XK_BackSpace, XCF_Cancel);
    }
    else {
-      add_binding(BUTTON2, XCF_Text_Return);
-      add_binding(BUTTON2, XCF_Select);
-      add_binding(HOLD | BUTTON2, XCF_SelectBox);
-      add_binding(BUTTON2, XCF_Finish_Element);
-      add_binding(BUTTON2, XCF_Finish_Copy);
-      add_binding(BUTTON3, XCF_Cancel_Last);
-      add_binding(BUTTON3, XCF_Cancel);
+      add_binding(ALL_WINDOWS, BUTTON2, XCF_Text_Return);
+      add_binding(ALL_WINDOWS, SHIFT | BUTTON1, XCF_Text_Return);
+      add_binding(ALL_WINDOWS, BUTTON2, XCF_Select);
+      add_binding(ALL_WINDOWS, SHIFT | BUTTON1, XCF_Select);
+      add_binding(ALL_WINDOWS, HOLD | BUTTON2, XCF_SelectBox);
+      add_binding(ALL_WINDOWS, SHIFT | HOLD | BUTTON1, XCF_Select);
+      add_binding(ALL_WINDOWS, BUTTON2, XCF_Finish_Element);
+      add_binding(ALL_WINDOWS, SHIFT | BUTTON1, XCF_Finish_Element);
+      add_binding(ALL_WINDOWS, BUTTON2, XCF_Finish_Copy);
+      add_binding(ALL_WINDOWS, SHIFT | BUTTON1, XCF_Finish_Copy);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Cancel_Last);
+      add_binding(ALL_WINDOWS, BUTTON3, XCF_Cancel);
    }
 }
 
@@ -730,8 +808,10 @@ static int button1mode = XCF_Wire;
 
 void mode_rebinding(int newmode, int newvalue)
 {
-   remove_binding(BUTTON1, button1mode);
-   add_vbinding(BUTTON1, newmode, (short)newvalue);
+   xcWidget window = areawin->area;
+
+   remove_binding(window, BUTTON1, button1mode);
+   add_vbinding(window, BUTTON1, newmode, (short)newvalue);
    button1mode = newmode;
    toolcursor(newmode);
 }
@@ -745,14 +825,15 @@ void mode_tempbinding(int newmode, int newvalue)
 {
    short saveval;
    XPoint cpos;
+   xcWidget window = areawin->area;
 
-   if (isnbound(BUTTON1, button1mode, &saveval)) {
-      remove_binding(BUTTON1, button1mode);
-      add_vbinding(BUTTON1, newmode, (short)newvalue);
+   if (boundfunction(window, BUTTON1, &saveval) == button1mode) {
+      remove_binding(window, BUTTON1, button1mode);
+      add_vbinding(window, BUTTON1, newmode, (short)newvalue);
       cpos = UGetCursor();
       eventdispatch(BUTTON1, (int)cpos.x, (int)cpos.y);
-      remove_binding(BUTTON1, newmode);
-      add_vbinding(BUTTON1, button1mode, saveval);
+      remove_binding(window, BUTTON1, newmode);
+      add_vbinding(window, BUTTON1, button1mode, saveval);
    }
    else
       fprintf(stderr, "Error: No such button1 binding %s\n",
