@@ -59,7 +59,7 @@ void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
       areawin->filter |= bitwise;
 
 #ifndef TCL_WRAPPER
-   toggle(w, &bval, calldata);
+   toggle(w, (pointertype)&bval, calldata);
 #endif
 }
 
@@ -250,7 +250,7 @@ Boolean pathselect(genericptr *curgen, short class, float range)
    /*		is an empirical result.					   */
    /*----------------------------------------------------------------------*/
 
-   float	wirelim = 2 + range / (*areawin->vscale + 0.05);
+   float	wirelim = 2 + range / (areawin->vscale + 0.05);
    long		sqrwirelim = (int)(wirelim * wirelim);
 
    long		newdist;
@@ -589,7 +589,6 @@ void selectarea()
 		  slab->scale, slab->rotation);
 
 	   if (slab->pin) {
-	      if (!areawin->schemon) continue;
 	      for (j = 0; j < 4; j++) 
 		 pinadjust(slab->justify, &(newboxpts[j].x),
 			&(newboxpts[j].y), -1);
